

set 2Ddirectory $env(HOME)
set 2Dfilename ""

set 2Ddimmenu [list 64 128 256 512 1024]
set 2Doffsetmenu [list 0 512 1024 2048 4096 8192]
set 2Dshiftmenu [list 0 1 2 3 4 5 6 7 8]

set 2Dgroupmenu {}
for {set i 0} {$i < 256} {incr i} {lappend 2Dgroupmenu $i}

set 2Ditemmenu {}
for {set i 0} {$i < 64} {incr i} {lappend 2Ditemmenu $i}


proc create_2D_spectra {} {
    global SpectrumServer 2Dlist
    global Options

    foreach item $2Dlist {

         set name       [lindex $item 0]
         set xdim       [lindex $item 1]
         set ydim       [lindex $item 2]
         set xlabel     [lindex $item 3]
         set xoffset    [lindex $item 4]
         set xshift     [lindex $item 5]
         set ylabel     [lindex $item 6]
         set yoffset    [lindex $item 7]
         set yshift     [lindex $item 8]
  
           set-footer "creating spectra for $name"

           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $xdim :range2 $ydim :type 5 :name '$name' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum $name"
              clear-footer
              error $m
           } else {
               execute-command 17 "Mapping spectrum $name"  "$name $xdim $ydim [expr $xlabel] $xoffset $xshift [expr $ylabel] $yoffset $yshift"
           }
    }
}


#    remaining procedures are for the 2D histogram configuration window

proc 2D_Histogram_Configuration {} {
    global 2Dlist

    2Ddefine
    if {[llength $2Dlist] == 0} then {2Dload} else {2Dselect}
}

proc Create_2D_Histograms {} {

    execute-command 16 "Initialise 2D Histogramming"
    create_2D_spectra
}

if {![info exists 2Dlist]} {
set 2Dlist {}
set 2Dselection 0

set 2Doptions(name)       "Sample2DHistogram"
set 2Doptions(xdim)       64
set 2Doptions(ydim)       64
set 2Doptions(xlabel)     0x0001
set 2Doptions(xoffset)    0
set 2Doptions(xshift)     0
set 2Doptions(ylabel)     0x0101
set 2Doptions(yoffset)    0
set 2Doptions(yshift)     0

set 2dxgroup [expr $2Doptions(xlabel) & 255]; set 2dxitem [expr ($2Doptions(xlabel) >> 8) & 255]
set 2dygroup [expr $2Doptions(ylabel) & 255]; set 2dyitem [expr ($2Doptions(ylabel) >> 8) & 255]

}

proc 2Dformat l {
  set ll {}
  foreach f $l {lappend ll [format "%32s %8s %8s   0x%04x %8s %8s   0x%04x %8s %8s" [lindex $f 0] [lindex $f 1] [lindex $f 2] \
     [lindex $f 3] [lindex $f 4] [lindex $f 5] [lindex $f 6] [lindex $f 7] [lindex $f 8]]}
  return $ll
}


proc 2Ddefine {}  {
    global env 2Ddimmenu 2Doffsetmenu 2Dshiftmenu 2Dgroupmenu 2Ditemmenu

set cols 110

  :frame "[self frame-name].2dhistograms" :label "Define 2D Histograms" :show-footer T  :resize F :toplevel T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

   :panel 2DCONFIGURATION  :layout T :padx 10 :pady 10
     :layout V; :row-gap 10; :col-gap 10

      :list 2DM    :rows  1 :cols $cols :deaf T :scroll N \
         :strings [list [format "%32s %8s %8s %8s %8s %8s %8s %8s %8s" "name" "xdim" "ydim" "xlabel" "xoffset" "xshift" "ylabel" "yoffset" "yshift"]]
      :list 2DLIST :rows 12 :cols $cols :exclusive T :strings [list ""] :command 2Doption

   :panel 2DOPTIONS :layout T :padx 10
     :layout H; :row-gap 10; :col-gap 0

      :text 2DNAME            :label "Name:"        :w 32 :command 2Doption
      :hskip 15
      :text 2DXDIM            :label "X dimension:" :w 8  :command 2Doption
      :choice-stack 2DXDIMM   :label ""  :menu $2Ddimmenu :command 2Doption
      :hskip 15
      :text 2DYDIM            :label "Y dimension:" :w 8  :command 2Doption
      :choice-stack 2DYDIMM   :label ""  :menu $2Ddimmenu :command 2Doption
      :next-row T
      :text 2DXSOURCE         :label "X source"     :w 32 :command 2Doption  :inactive T
      :menubutton 2DXNAME     :label "select" :menu "" :command 2Doption
      :hskip 10
      :text 2DXLABEL          :label "X Label:"     :w 8  :command 2Doption
      :hskip 16
      :choice-stack 2DXGROUP  :label "X group"  :strings $2Dgroupmenu :command 2Doption
      :hskip 16
      :choice-stack 2DXITEM   :label "X item"   :strings $2Ditemmenu  :command 2Doption
      :next-row T
      :text 2DYSOURCE         :label "Y source"     :w 32 :command 2Doption  :inactive T
      :menubutton 2DYNAME     :label "select" :menu "" :command 2Doption
      :hskip 12
      :text 2DYLABEL          :label "Y Label:"     :w 8  :command 2Doption
      :hskip 16
      :choice-stack 2DYGROUP  :label "Y group"  :strings $2Dgroupmenu :command 2Doption
      :hskip 18
      :choice-stack 2DYITEM   :label "Y item"   :strings $2Ditemmenu  :command 2Doption
      :next-row T
      :text 2DXOFFSET         :label "X Offset:"    :w 8  :command 2Doption
      :choice-stack 2DXOFFSETM   :label ""  :menu $2Doffsetmenu :command 2Doption
      :hskip 25
      :text 2DXSHIFT          :label "X Shift :"    :w 8  :command 2Doption
      :choice-stack 2DXSHIFTM    :label ""  :menu $2Dshiftmenu :command 2Doption
      :hskip 15
      :text 2DYOFFSET         :label "Y Offset:"    :w 8  :command 2Doption
      :choice-stack 2DYOFFSETM   :label ""  :menu $2Doffsetmenu :command 2Doption
      :hskip 25
      :text 2DYSHIFT          :label "Y Shift :"    :w 8  :command 2Doption
      :choice-stack 2DYSHIFTM    :label ""  :menu $2Dshiftmenu :command 2Doption

   :panel 2DCLICK  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 15

      :button 2DADD     :label Add      :command 2Dadd
      :button 2DREPLACE :label Replace  :command 2Dreplace
      :button 2DDELETE  :label Delete   :command 2Ddelete
      :button 2DCLEAR   :label Clear    :command 2Dclear
     :next-row T
      :col-gap 5
      :button 2DSAVE :label "Save configuration"        :command 2Dsave
      :button 2DREST :label "Restore configuration"     :command 2Drestore
      :button 2DDONE :label "Save configuration and finish"     :command 2Ddone
      :button 2DQUIT :label "Quit without saving configuration" :command 2Dquit

    2Dmenugen

     :show T
}

proc 2Dquit {} {frame-close "[self frame-name].2dhistograms"}

proc 2Ddone {} {2Dsave; 2Dquit}

proc 2Dupdate {} {
    global 2Dlist 2Doptions 2Dselection 2Ddimmenu 2Doffsetmenu 2Dshiftmenu 2Dgroupmenu 2Ditemmenu

     :modify   2DNAME     :value $2Doptions(name)
     :modify   2DXDIM     :value $2Doptions(xdim)
     :modify   2DXDIMM    :value [lsearch $2Ddimmenu $2Doptions(xdim)]
     :modify   2DYDIM     :value $2Doptions(ydim)
     :modify   2DYDIMM    :value [lsearch $2Ddimmenu $2Doptions(ydim)]
     :modify   2DXLABEL   :value [format "0x%04x" $2Doptions(xlabel)]
     :modify   2DYLABEL   :value [format "0x%04x" $2Doptions(ylabel)]
     :modify   2DXOFFSET  :value $2Doptions(xoffset)
     :modify   2DXOFFSETM :value [lsearch $2Doffsetmenu $2Doptions(xoffset)]
     :modify   2DXSHIFT   :value $2Doptions(xshift)
     :modify   2DXSHIFTM  :value [lsearch $2Dshiftmenu $2Doptions(xshift)]
     :modify   2DYOFFSET  :value $2Doptions(yoffset)
     :modify   2DYOFFSETM :value [lsearch $2Doffsetmenu $2Doptions(yoffset)]
     :modify   2DYSHIFT   :value $2Doptions(yshift)
     :modify   2DYSHIFTM  :value [lsearch $2Dshiftmenu $2Doptions(yshift)]

     :modify  2DXGROUP   :value [lsearch $2Dgroupmenu [expr $2Doptions(xlabel) & 255]]
     :modify  2DYGROUP   :value [lsearch $2Dgroupmenu [expr $2Doptions(ylabel) & 255]]
     :modify  2DXITEM    :value [lsearch $2Ditemmenu [expr ($2Doptions(xlabel) >> 8) & 255]]
     :modify  2DYITEM    :value [lsearch $2Ditemmenu [expr ($2Doptions(ylabel) >> 8) & 255]]

    :modify 2DLIST :show F :empty T :strings "[2Dformat $2Dlist]" :show T
    :modify 2DLIST :select $2Dselection
    :modify 2DLIST :see 0
}

proc 2Doption {W vl args} {
   global VXIsource
   global 2Doptions 2Dselection 2dxgroup 2dxitem  2dygroup 2dyitem

    switch $W {
        2DNAME     {set 2Doptions(name) $vl}
        2DXDIM     {set 2Doptions(xdim) $vl}
        2DXDIMM    {set 2Doptions(xdim) [first $args]}
        2DYDIM     {set 2Doptions(ydim) $vl}
        2DYDIMM    {set 2Doptions(ydim) [first $args]}
        2DXLABEL   {set 2Doptions(xlabel) $vl; set 2dxgroup [expr $2Doptions(xlabel) & 255]; set 2dxitem [expr ($2Doptions(xlabel) >> 8) & 255]}
        2DYLABEL   {set 2Doptions(ylabel) $vl; set 2dygroup [expr $2Doptions(ylabel) & 255]; set 2dyitem [expr ($2Doptions(ylabel) >> 8) & 255]}
        2DXOFFSET  {set 2Doptions(xoffset) $vl}
        2DXOFFSETM {set 2Doptions(xoffset) [first $args]}
        2DXSHIFT   {set 2Doptions(xshift) $vl}
        2DXSHIFTM  {set 2Doptions(xshift) [first $args]}
        2DYOFFSET  {set 2Doptions(yoffset) $vl}
        2DYOFFSETM {set 2Doptions(yoffset) [first $args]}
        2DYSHIFT   {set 2Doptions(yshift) $vl}
        2DYSHIFTM  {set 2Doptions(yshift) [first $args]}
        2DLIST     {set 2Dselection [first $args]; 2Dselect}
        2DXGROUP   {set 2dxgroup $vl; set 2Doptions(xlabel) [expr ($2dxitem << 8) + $2dxgroup]}
        2DXITEM    {set 2dxitem $vl;  set 2Doptions(xlabel) [expr ($2dxitem << 8) + $2dxgroup]}
        2DYGROUP   {set 2dygroup $vl;  set 2Doptions(ylabel) [expr ($2dyitem << 8) + $2dygroup]}
        2DYITEM    {set 2dyitem $vl;  set 2Doptions(ylabel) [expr ($2dyitem << 8) + $2dygroup]}
        2DXSOURCE  {}
        2DXNAME    {set 2Doptions(xlabel) $VXIsource($vl); :modify 2DXSOURCE :inactive F :value $vl :inactive T;  \
            set 2dxgroup [expr $2Doptions(xlabel) & 255]; set 2dxitem [expr ($2Doptions(xlabel) >> 8) & 255]}
        2DYSOURCE  {}
        2DYNAME    {set 2Doptions(ylabel) $VXIsource($vl); :modify 2DYSOURCE :inactive F :value $vl :inactive T;  \
            set 2dxgroup [expr $2Doptions(xlabel) & 255]; set 2dxitem [expr ($2Doptions(xlabel) >> 8) & 255]}
    }

    2Dupdate
}

proc 2Dadd {args} {
    global 2Dlist 2Doptions
    lappend 2Dlist [list $2Doptions(name) $2Doptions(xdim) $2Doptions(ydim)  \
         $2Doptions(xlabel) $2Doptions(xoffset) $2Doptions(xshift)  \
         $2Doptions(ylabel) $2Doptions(yoffset) $2Doptions(yshift)]
    2Dupdate
}

proc 2Dreplace {args} {
    global 2Dlist 2Doptions 2Dselection
    set 2Dlist [lreplace $2Dlist $2Dselection $2Dselection \
       [list $2Doptions(name) $2Doptions(xdim) $2Doptions(ydim)  \
          $2Doptions(xlabel) $2Doptions(xoffset) $2Doptions(xshift)  \
          $2Doptions(ylabel) $2Doptions(yoffset) $2Doptions(yshift)]]

    2Dupdate
}

proc 2Dselect {} {
    global 2Dlist 2Doptions 2Dselection 2dxgroup 2dxitem 2dygroup 2dyitem

         set 2Doptions(name)       [lindex [lindex $2Dlist $2Dselection] 0]
         set 2Doptions(xdim)       [lindex [lindex $2Dlist $2Dselection] 1]
         set 2Doptions(ydim)       [lindex [lindex $2Dlist $2Dselection] 2]
         set 2Doptions(xlabel)     [lindex [lindex $2Dlist $2Dselection] 3]
         set 2Doptions(xoffset)    [lindex [lindex $2Dlist $2Dselection] 4]
         set 2Doptions(xshift)     [lindex [lindex $2Dlist $2Dselection] 5]
         set 2Doptions(ylabel)     [lindex [lindex $2Dlist $2Dselection] 6]
         set 2Doptions(yoffset)    [lindex [lindex $2Dlist $2Dselection] 7]
         set 2Doptions(yshift)     [lindex [lindex $2Dlist $2Dselection] 8]

       set 2dxgroup [expr $2Doptions(xlabel) & 255]; set 2dxitem [expr ($2Doptions(xlabel) >> 8) & 255]
       set 2dygroup [expr $2Doptions(ylabel) & 255]; set 2dyitem [expr ($2Doptions(ylabel) >> 8) & 255]

    2Dupdate
    :modify 2DXSOURCE :value ""
    :modify 2DYSOURCE :value ""
}

proc 2Ddelete {args} {
    global 2Dlist 2Doptions 2Dselection
    set 2Dlist [lreplace $2Dlist $2Dselection $2Dselection]
    2Dupdate
}

proc 2Dclear {args} {
    global 2Dlist

    set 2Dlist {}

    2Dupdate
    :modify 2DXSOURCE :value ""
    :modify 2DYSOURCE :value ""
}

proc 2Dsave {}  {
   global env 2Dlist  2Ddirectory 2Dfilename

#    data has been modified  - save to file

    set file1 [file join $env(EXPERIMENTBASE) 2DHistograms]
    set z [catch {open $file1 w} f]
    if {$z != 0} then {midas-warning "Unable to write file $file1\n$f"; return}

    foreach item $2Dlist {puts $f "$item"}
    close $f

    set-footer "$file1 updated"

    option add *Canvas.background  #c0c020
    set file2 [tk_getSaveFile -parent [self frame-name] \
         -title "2D Histogram Configuration"  \
	 -initialdir $2Ddirectory  -initialfile $2Dfilename]
    option add *Canvas.background  $env(tkcanvasbackground)
    if {$file2 == ""} then {return}

    set z [ catch {open $file2 w} f ]
    if { $z != 0 } then {midas-warning "Unable to open file $file2\n$f"; return}

    puts $f "#  format is  \"%32s %8d %8d   0x%04x %8d %8d   0x%04x %8d %8d\"   name xdim ydim xlabel xoffset xshift ylabel yoffset yshift"

    foreach item $2Dlist {puts $f "$item"}
    close $f

    set-footer "$file1 and $file2 updated"

    set 2Ddirectory [file dirname $file2]
    set 2Dfilename [file tail $file2]
}

proc 2Drestore {}  {
   global env 2Doptions 2Dlist  2Ddirectory 2Dfilename 2dxgroup 2dxitem 2dygroup 2dyitem

    option add *Canvas.background  #c0c020
    set file2 [tk_getOpenFile -parent [self frame-name] \
         -title "2D Histogram Configuration"  \
	 -initialdir $2Ddirectory  -initialfile $2Dfilename]
    option add *Canvas.background  $env(tkcanvasbackground)
    if {$file2 == ""} then {return}

    set z [ catch {open $file2 r} f ]
    if { $z != 0 } then {midas-warning "Unable to open file $file2\n$f"; return}

    set configuration [split [read $f] \n]
    close $f

    2Dclear

    foreach item $configuration {

       if {[string range $item 0 0] == "#"} {continue}
       if {[llength $item] != 9} {continue}

       set 2Doptions(name)       [lindex $item 0]
       set 2Doptions(xdim)       [lindex $item 1]
       set 2Doptions(ydim)       [lindex $item 2]
       set 2Doptions(xlabel)     [lindex $item 3]
       set 2Doptions(xoffset)    [lindex $item 4]
       set 2Doptions(xshift)     [lindex $item 5]
       set 2Doptions(ylabel)     [lindex $item 6]
       set 2Doptions(yoffset)    [lindex $item 7]
       set 2Doptions(yshift)     [lindex $item 8]

       set 2dxgroup [expr $2Doptions(xlabel) & 255]; set 2dxitem [expr ($2Doptions(xlabel) >> 8) & 255]
       set 2dygroup [expr $2Doptions(ylabel) & 255]; set 2dyitem [expr ($2Doptions(ylabel) >> 8) & 255]

       2Dadd
    }

    set-footer "$file2 restored"

    set file1 [file join $env(EXPERIMENTBASE) 2DHistograms]
    set z [catch {open $file1 w} f]
    if {$z != 0} then {midas-warning "Unable to write file $file1\n$f"; return}

    foreach item $2Dlist {puts $f "$item"}
    close $f

    set 2Ddirectory [file dirname $file2]
    set 2Dfilename [file tail $file2]

    set-footer "$file2 and $file1 restored"
}


proc 2Dload {}  {
   global env 2Dlist 2Doptions  2dxgroup 2dxitem 2dygroup 2dyitem

#    load configuration from file

    2Dclear

    set file [file join $env(EXPERIMENTBASE) 2DHistograms]
    set z [catch {open $file r} f]
    if {$z != 0} then {2Dsave; return}

    set configuration [split [read $f] \n]
    close $f

    foreach item $configuration {
       if {[llength $item] == 9} {
         set 2Doptions(name)       [lindex $item 0]
         set 2Doptions(xdim)       [lindex $item 1]
         set 2Doptions(ydim)       [lindex $item 2]
         set 2Doptions(xlabel)     [lindex $item 3]
         set 2Doptions(xoffset)    [lindex $item 4]
         set 2Doptions(xshift)     [lindex $item 5]
         set 2Doptions(ylabel)     [lindex $item 6]
         set 2Doptions(yoffset)    [lindex $item 7]
         set 2Doptions(yshift)     [lindex $item 8]

       set 2dxgroup [expr $2Doptions(xlabel) & 255]; set 2dxitem [expr ($2Doptions(xlabel) >> 8) & 255]
       set 2dygroup [expr $2Doptions(ylabel) & 255]; set 2dyitem [expr ($2Doptions(ylabel) >> 8) & 255]

         2Dadd
       }
    }

    set-footer "$file restored"
}


proc get-VXI-config {} {
    global env VXIconfig

   set Configdir [file join $env(MIDASAPPLBASE) config]
   set experiment [file tail $env(EXPERIMENTBASE)]
   set fname [file join $Configdir [set experiment]_Current_VXI_Configuration]

   set VXIconfig "#"

   if {![file exists $fname]} {
       set fname [file join $Configdir Default_VXI_Configuration]
       if {![file exists $fname]} {
           set fname [file join $env(MIDASBASE) vxi tcl Default_VXI_Configuration]
       }
   }

   set z [catch {open $fname r} f]
   if {$z != 0} then {midas-warning "Unable to read file $fname\n$f"; return}
   set VXIconfig [split [read $f] \n]
   close $f
}

proc 2Dmenugen {} {
    global VXIconfig VXIsource

    get-VXI-config

    catch {unset level1}
    catch {unset level2}
    foreach item $VXIconfig {
        if {$item == ""} {continue}
        if {[string range [first $item] 0 0] == "#"} {continue}
        set VXIsource([first $item]) [lindex $item 3]
        set m [first [split [first $item] .]]
        switch -glob $m {
            Trigger*   {set t $m}
            ADC*       {set t [string range $m 0 7]}
            default    {set t [first [split $m 0123456789]]}
        }
        if {![info exists level1($t)] || [lsearch $level1($t) $m] == -1} {lappend level1($t) $m}
        lappend level2($m) [first $item]
    }

    catch {unset menu}
    foreach item1 [lsort -dictionary [array names level1]] {  ;#  eg item1= ECC
        set menu1 [list $item1]
        foreach item2 $level1($item1) {                       ;#  eg item2= ECC0 ECC1 ECC2 etc
            set menu2 [list $item2]
            foreach item3 $level2($item2) {                   ;#  eg  item3= ECC0.A.6MeV etc
                lappend menu2 $item3
            }
            lappend menu1 [lsort -dictionary $menu2]
        }
        lappend menu [lsort -dictionary $menu1]
    }

    :modify 2DXNAME :menu $menu
    :modify 2DYNAME :menu $menu
}


proc :modify_menubutton {win name args} {
  global debug _rows
  set __rows 10

#    options -   :strings ?

  set args [lindex $args 0]
  if {$debug} then {insert-debug ":modify_menubutton called with  $win $name $args"}

  if ([info exists _rows($name)])  {set __rows $_rows($name)}

  set i 0
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set st [lindex $args [expr $i + 1]]
    switch $p {

      :rows {set __rows $p; set _rows($name) $p}
      :deaf {}
      :inactive  {
        switch $st {
          T {$win configure -state disabled}
          F {$win configure -state normal}
          default {puts stderr ":modify_menubutton  :inactive  - unknown option $v"}
        }
      }
      :label {$win configure -text "$st"}
      :strings  {
        $win.menu delete 0 end
        set ii 0
        set jj 0
        foreach s $st {
          if {$jj == $__rows} then {
             $win.menu add command -label "$s" -columnbreak 1 \
                -command ":menubutton-event $win $name \"$s\" $ii"
             set jj 0
          } else {
             $win.menu add command -label "$s"  \
                -command ":menubutton-event $win $name \"$s\" $ii"
          }
          lappend _$name.menu "$s"
          incr ii
          incr jj
        }
      }
      :menu {
        $win.menu delete 0 end
        set ii 0
        set jj 0
        set kk 0
        foreach s $st {
          if {[llength $s] <= 1} then {
            $win.menu add command -label "$s" \
               -command ":menubutton-event $win $name \"$s\" $kk"
            incr kk
          } else {
            incr ii
            set c [first $s]
            $win.menu add cascade -label "$c" -menu $win.menu.c$ii
            set s [lreplace $s 0 0]
            catch {menu $win.menu.c$ii -tearoff true}
            :_set-font $win.menu.c$ii menu 0 N
            $win.menu.c$ii delete 0 end
            foreach ss $s {
               if {[llength $ss] <= 1} then {
                   $win.menu.c$ii add command -label "$ss" \
                      -command ":menubutton-event $win $name \"$ss\" $kk"
                   incr kk
               } else {
                   incr jj
                   set cc [first $ss]
                   $win.menu.c$ii add cascade -label "$cc" -menu $win.menu.c$ii.c$jj
                   set ss [lreplace $ss 0 0]
                   catch {menu $win.menu.c$ii.c$jj -tearoff true}
                   :_set-font $win.menu.c$ii.c$jj menu 0 N
                   $win.menu.c$ii.c$jj delete 0 end
                   foreach sss $ss {
                      $win.menu.c$ii.c$jj add command -label "$sss" \
                          -command ":menubutton-event $win $name \"$sss\" $kk"
                      incr kk
                   }
               }
            }
         }
       }
      }
      default {puts stderr ":modify_menubutton - unknown arg $p"}
    }
    set i [expr $i + 2]
  }

}
