
proc Set_VME_Options {} {
    global Options

   load_Options

   if {![info exists Options(vme.channels)]} {set Options(vme.channels) 8192}
   if {![info exists Options(vme.offset)]} {set Options(vme.offset) 0}
   if {![info exists Options(vme.shift)]} {set Options(vme.shift) 3}

   update_Options
}

proc Dump_Raw_VME_Data_VME {} {

   set info "VME\n"

   execute-command 19 "Getting Raw Data for VME" 5 2
   append data [read-register NetVar.EXEC.Response]

     foreach word $data {
         set word 0x$word
         if {$word == 0xaa5555aa} then {
             append info "\n$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "\n$word - end of event\n"
             continue
         }
         append info "$word "
     }

    append info "\n"

   return $info
}

proc create_VME_spectra {} {
    global SpectrumServer VMElist
    global Options

    foreach item $VMElist {
       set width [third $item]
       set group [fourth $item]
       set item [fifth $item]

       foreach j [list 0 1] {
  
           if {$width == 16 && $j == 1} {break}
  
           set-footer "creating spectra for group$group.item$item"

           set spectrumname [SpectrumName  group$group.item$item]
           set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(vme.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
           if {$z!=0 && $m==0x30004} then {set z 0}
           if {$z!=0} then {
              display-eg-error-report self $m "Attempting to create spectrum group$group.item$item"
              clear-footer
              error $m
           } else {
               map_spectrum  "$spectrumname $group $item $Options(vme.offset) $Options(vme.shift)"
           }
           incr item
        }
    }
}

proc define_VME_config {} {
    global VMElist

   global RDOUTEnable; if {$RDOUTEnable == 0} then {return}

    if {[llength $VMElist] == 0} then {VMEdefine; VMEquit; VMEload}
    if {[llength $VMElist] == 0} then {return}

#    format of configuration record is  Address_Space VME_Address Data_Width Group Item
#    Where Address_Space is  16; 24 or 32
#          Data_Width is 16 or 32

    foreach item $VMElist {
       if {[llength $item] == 5} {
         execute-command 14 "Define VME Data Source Configuration" "$item" 
       }
    }
}

#    remaining procedures are for the VME data source configuration window

proc Configure_VME_Data_Sources {} {
    global VMElist

    VMEdefine
    if {[llength $VMElist] == 0} then {VMEload} else {VMEselect}
}

if {![info exists VMElist]} {
set VMElist {}
set VMEselection 0

set VMEoptions(address) 0x20000000
set VMEoptions(space) 32
set VMEoptions(width) 16
set VMEoptions(group) 1
set VMEoptions(item) 0
}

if {![info exists groups]} then {
   set groups {}
   for {set i 0} {$i <= 50} {incr i} {lappend groups $i}
}

if {![info exists items]} then {
   set items {}
   for {set i 0} {$i < 64} {incr i} {lappend items $i}
}

proc VMEformat l {
  set ll {}
  foreach f $l {lappend ll [format "%8s %12s %8s %8s %8s" [first $f] \
    [second $f] [third $f] [fourth $f] [fifth $f]]}
  return $ll
}


proc VMEdefine {}  {
    global env groups items

set cols 50

  :frame "[self frame-name].datasources" :label "Define VME Data Sources" :show-footer T  :resize F :toplevel T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

   :panel VMECONFIGURATION  :layout T :padx 10 :pady 10
     :layout V; :row-gap 10; :col-gap 10

      :list VMEM    :rows  1 :cols $cols :deaf T :scroll N \
         :strings [list [format "%8s %12s %8s %8s %8s" "space" "address" "width" "group" "item"]]
      :list VMELIST :rows 12 :cols $cols :exclusive T :strings [list ""] :command VMEoption

   :panel VMEOPTIONS :layout T :padx 10
     :layout H; :row-gap 10; :col-gap 15

      :choice-stack VMESPACE   :label "Address Space:"  :strings [list A16 A24 A32] :command VMEoption
      :text VMEADDRESS         :label "Address Offset:" :w 10 :value 0x0 :command VMEoption
     :next-row T
      :choice-stack VMEWIDTH  :label "Data Width:"  :strings [list D16 D32]      :command VMEoption
      :choice-stack VMEGROUP  :label Group:  :exclusive T  :strings "$groups"    :command VMEoption
      :choice-stack VMEITEM   :label Item:   :exclusive T  :strings "$items"     :command VMEoption

   :panel VMECLICK  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 15

      :button VMEADD     :label Add      :command VMEadd
      :button VMEREPLACE :label Replace  :command VMEreplace
      :button VMEDELETE  :label Delete   :command VMEdelete
      :button VMECLEAR   :label Clear    :command VMEclear
     :next-row T
      :col-gap 5
      :button VMEDONE :label "Save configuration and finish"     :command VMEdone
      :button VMEQUIT :label "Quit without saving configuration" :command VMEquit

     :show T
}

proc VMEquit {} {frame-close "[self frame-name].datasources"}

proc VMEdone {} {VMEsave; VMEquit}

proc VMEupdate {} {
    global VMElist VMEoptions VMEselection

    :modify VMESPACE    :value [lsearch [list 16 24 32] $VMEoptions(space)]
    :modify VMEADDRESS  :value $VMEoptions(address)
    :modify VMEWIDTH    :value [lsearch [list 16 32] $VMEoptions(width)]
    :modify VMEGROUP    :value $VMEoptions(group)
    :modify VMEITEM     :value $VMEoptions(item)

    :modify VMELIST :show F :empty T :strings "[VMEformat $VMElist]" :show T
    :modify VMELIST :select $VMEselection
    :modify VMELIST :see 0
}

proc VMEoption {W vl args} {
   global VMEoptions VMEselection

    switch $W {
        VMESPACE    {set VMEoptions(space) [lindex [list 16 24 32] $vl]}
        VMEADDRESS  {set VMEoptions(address) $vl}
        VMEWIDTH    {set VMEoptions(width) [lindex [list 16 32] $vl]}
        VMEGROUP    {set VMEoptions(group) [first $args]}
        VMEITEM     {set VMEoptions(item) [first $args]}
        VMELIST     {set VMEselection [first $args]; VMEselect}
    }
}

proc VMEadd {args} {
    global VMElist VMEoptions
    lappend VMElist [list $VMEoptions(space) $VMEoptions(address) $VMEoptions(width) $VMEoptions(group) $VMEoptions(item)]
    VMEupdate
}

proc VMEreplace {args} {
    global VMElist VMEoptions VMEselection
    set VMElist [lreplace $VMElist $VMEselection $VMEselection \
       [list $VMEoptions(space) $VMEoptions(address) $VMEoptions(width) $VMEoptions(group) $VMEoptions(item)]]
    VMEupdate
}

proc VMEselect {} {
    global VMElist VMEoptions VMEselection

    set VMEoptions(space)   [first [lindex $VMElist $VMEselection]]
    set VMEoptions(address) [second [lindex $VMElist $VMEselection]]
    set VMEoptions(width)   [third [lindex $VMElist $VMEselection]]
    set VMEoptions(group)   [fourth [lindex $VMElist $VMEselection]]
    set VMEoptions(item)    [fifth [lindex $VMElist $VMEselection]]

    VMEupdate
}

proc VMEdelete {args} {
    global VMElist VMEoptions VMEselection
    set VMElist [lreplace $VMElist $VMEselection $VMEselection]
    VMEupdate
}

proc VMEclear {args} {
    global VMElist
    set VMElist {}
    VMEupdate
}

proc VMEsave {}  {
   global env VMElist

#    data has been modified  - save to file

    set file [file join $env(EXPERIMENTBASE) VMEDataSources]
    set z [catch {open $file w} f]
    if {$z != 0} then {midas-warning "Unable to write file $file\n$f"; return}

    foreach item $VMElist {
        puts $f "$item"
    }
    close $f
    set-footer "$file updated"
}

proc VMEload {}  {
   global env VMElist VMEoptions

#    load configuration from file

    VMEclear

    set file [file join $env(EXPERIMENTBASE) VMEDataSources]
    set z [catch {open $file r} f]
    if {$z != 0} then {VMEsave; return}

    set configuration [split [read $f] \n]
    close $f

    foreach item $configuration {
       if {[llength $item] == 5} {
         set VMEoptions(space)   [first $item]
         set VMEoptions(address) [second $item]
         set VMEoptions(width)   [third $item]
         set VMEoptions(group)   [fourth $item]
         set VMEoptions(item)    [fifth $item]

         VMEadd
       }
    }

    set-footer "$file restored"
}
